from plow.core.poll import Poll
import re
import json
import arrow


class Dhwp(Poll):
    def run(self, **kwargs):
        for person in self.customer_info.list_persons():
            p = self.customer_info.get_person(person)
            for d in p.get_domains():
                created = None
                try:
                    created = arrow.get(d.created)
                except Exception as e:
                    continue

                if created < arrow.utcnow().to('US/Pacific').shift(months=-6):
                    continue

                if hasattr(d, 'wordpress') and 'path' in d.wordpress and hasattr(d, 'user') and hasattr(d, 'home'):
                    for wp_type in (['plugin', 'theme', 'post']):
                        result = self.do_command(
                            self.wp_cli_cmd(
                                d.wordpress['path'],
                                ('%s list' % wp_type)
                            ),
                            machine=d.home,
                            timeout=10,
                            user=d.user
                        )
                        if hasattr(self, '_format_result_%s' % wp_type):
                            try:
                                result = getattr(
                                    self, '_format_result_%s' % wp_type)(result)
                            except Exception as e:
                                self.log.error(
                                    "failed to format result type %s" % wp_type)
                                continue

                        if result:
                            self.add_state(
                                name='wordpress_%s' % wp_type,
                                obj_type='domain',
                                obj_value=d.domain,
                                state=result
                            )

    def _format_result_post(self, result=[]):
        myresult = {p['post_status']: 0 for p in result}
        for post in result:
            myresult[post['post_status']] += 1
        return myresult

    def _format_result_plugin(self, result=[]):
        return self._list_to_dict([r for r in result if r['status'] == 'active'], key_name='name')

    def _format_result_theme(self, result=[]):
        return self._list_to_dict([r for r in result if r['status'] == 'active'], key_name='name')

    def _list_to_dict(self, mylist, key_name='name'):
        mydict = {}
        for item in mylist:
            mydict[item[key_name]] = {k: item[k]
                                      for k in item.keys() if k != key_name}
        return mydict

    def wp_cli_cmd(self, path, args, json=True):
        # SUDO creates a shitload of anoying logs on machines
        # Popen(user=..)
        cmd = 'wp --path=%s --skip-plugins --skip-themes %s' % (
            path, args)
        if json:
            cmd += ' --format=json'
        return cmd

    def process_results(self, result):
        json_data = None
        try:
            json_data = json.loads(result)
        except Exception as e:
            self.log.error("failed to parse wp cli output %s" % result)
        return json_data
