from plow.core.poll import Poll
import re
import json


class SAObj():
    def __init__(self, data={}):
        # only keep digits
        for k, v in data.items():
            v = ''.join(filter(str.isdigit, v))
            data[k] = v
        self.__dict__ = data


class Sa(Poll):
    def disable_load_check(self):
        return True

    def run(self, **kwargs):
        result = self.do_command("sa -m")
        for user, sa_obj in result.items():
            person = self.customer_info.person_by_user(user)
            if not person.no_person():
                # only keep people with more then 1 cpu  minute used
                if not sa_obj.cpu_min > 1:
                    continue
                self.metrics_state.gauge(
                    'sa_user_cpu_minutes',
                    tags=['user', 'person'],
                    tag_values=[user, person.person],
                    value=sa_obj.cpu_min
                )
                self.metrics_state.gauge(
                    'sa_user_cpu_re_seconds',
                    tags=['user', 'person'],
                    tag_values=[user, person.person],
                    value=sa_obj.re_time
                )

    def process_results(self, result):
        # uncomment for testing
        #        result = """jordan                             164      22.18re       3.40cp         0avio     82880k
        # wp_j2aq6p                             164      22.18re       3.40cp         0avio     82880k
        # """
        sa_dict = {}
        for line in result.splitlines():
            data_dict = {}
            (user, data_dict['num_calls'], data_dict['re_time'], data_dict['cpu_min'],
             data_dict['avio'], data_dict['cpu_avmem']) = line.split()
            sa_dict[user] = SAObj(data_dict)
        return sa_dict
