import os
# switch to ipaddress for puython3
from ipaddress import IPv4Network, IPv4Address
import json


def network_owners():
    path = os.path.dirname(os.path.abspath(__file__))
    rdap_path = path + '/../data_files/rdap'
    networks = {}
    for filename in os.listdir(rdap_path):
        with open(os.path.join(rdap_path, filename), encoding='utf-8') as json_file:
            rdap = json.load(json_file)

        handle = rdap['handle']
        networks[handle] = []
        for net in rdap['networks']:
            for cidr in net['cidr0_cidrs']:
                if 'v4prefix' in cidr:
                    networks[handle].append(IPv4Network(
                        "%s/%s" % (cidr['v4prefix'], cidr['length'])))
                # TODO Add v6 support later
    return networks


class NetworkOwner():
    def __init__(self):
        self.rdap_networks = network_owners()

    def get(self, ip):
        if ':' in ip:
            return None
        ip_obj = IPv4Address(ip)
        for owner, networks in self.rdap_networks.items():
            for net in networks:
                if ip_obj in net:
                    return owner
