from cement.utils.misc import minimal_logger
from apscheduler.schedulers.background import BackgroundScheduler
from apscheduler.executors.pool import ThreadPoolExecutor, ProcessPoolExecutor
import time
from pytz import timezone
import random
import importlib
import yaml
import arrow
import sys
import warnings


# TODO remove when these warnings are fixed in apscheduler
warnings.filterwarnings(
    "ignore",
    message="The normalize method is no longer necessary"
)
warnings.filterwarnings(
    "ignore",
    message="The zone attribute is specific"
)

commands = {
    'poll': 'plow.core.poller',
    'scrape': 'plow.core.scrapers'
}


class Scheduler():
    def __init__(self, logger, app_config=None):
        self.log = logger
        self.app_config = app_config
        self.debug = False
        if 'debug' in self.app_config:
            self.debug = True

    # TODO scheduler metrics
    def start(self, jobs_file):
        with open(jobs_file) as file:
            jobs = yaml.load(file, Loader=yaml.FullLoader)

        exec_pool = {
            'default': ThreadPoolExecutor(1),
            'poll': ThreadPoolExecutor(1),
            'scrape': ThreadPoolExecutor(3)
        }

        job_defaults = {
            'max_instances': 1
        }

        self.apscheduler = BackgroundScheduler(
            executors=exec_pool, job_defaults=job_defaults, timezone=timezone('US/Pacific'))

        for job, job_detail in jobs.items():
            # this dup logging?
            job_log = minimal_logger('plow-worker-' + job, debug=self.debug)
            try:
                cls_name = '.'.join(
                    (commands[job_detail['command']], job_detail['type']))
                cmd_cls = importlib.import_module(cls_name)
                job_obj = getattr(cmd_cls, job_detail['type'].capitalize())(
                    logger=job_log, app_config=self.app_config)
                job_args = {}

                if 'args' in job_detail:
                    job_args = job_detail['args']

                if 'schedule' in job_detail:
                    schedule = job_detail['schedule']

                    if 'trigger' not in schedule:
                        schedule['trigger'] = 'interval'

                    # backwards compat for old configs
                    if 'splay' in schedule:
                        schedule['jitter'] = schedule['splay']
                        del schedule['splay']

                    # Schedule timejob_detail['command']
                    self.log.info(
                        "Adding job %s, schedule %s" % (job, schedule))

                    self.apscheduler.add_job(
                        job_obj.do_run,
                        kwargs=job_args,
                        id=job,
                        executor=job_detail['command'],
                        **schedule
                    )
            except Exception as e:
                self.log.error("failed to schedule job : %s : %s" % (job, e))
                continue

        status_timer = 120
        last_status = arrow.now()
        self.apscheduler.start()
        while True:
            time.sleep(1)
            delta = arrow.now() - last_status
            if delta.total_seconds() >= status_timer:
                last_status = arrow.now()
                self.job_status()

        self.apscheduler.shutdown()
        sys.exit(1)

    def job_status(self):
        for job in self.apscheduler.get_jobs():
            self.log.info("job %s next run %s" % (
                job.id, job.next_run_time))
