from plow.core.scrape import Scrape
import re


class Auth(Scrape):
    def metrics(self):
        return {'sftp_counter', 'ssh_shell_counter', 'ftp_counter'}

    def parsers(self):
        return {
            'default': '',
            'sftp_counter': '%{SYSLOGTIMESTAMP:stamp} %{IPORHOST:host} %{SYSLOGPROG:prog}\: Starting session\: subsystem \'sftp\' for %{USER:user} from %{IPORHOST:remote_ip}',
            'ssh_shell_counter': '%{SYSLOGTIMESTAMP:stamp} %{IPORHOST:host} %{SYSLOGPROG:prog}\: Starting session: %{WORD:type} on pts/%{NUMBER:pts} for %{USER:user} from %{IPORHOST:remote_ip}',
            'ftp_counter': '%{SYSLOGTIMESTAMP:stamp} %{IPORHOST:host} %{SYSLOGPROG:prog}\: %{IPORHOST:host_ip} \(%{IPORHOST:remote_ip}\[%{IPORHOST:remote_host}\]\) - USER %{USER:user}%{DATA:something}?: %{GREEDYDATA:message}'
        }

    def is_webftp(self, match):
        if match['remote_ip'] == '69.163.136.91':
            return True
        return False

    def ftp_counter(self, matches):
        for user, um in self.matches_by_field('user', matches).items():
            login_success = [m for m in um if re.match(
                'Login successful', m['message'])]
            person = self.customer_info.person_by_user(user)
            self.metrics_state.counter(
                'user_login',
                tags=['person', 'user', 'type', 'result'],
                tag_values=[person.person, user, 'ftp', 'success'],
                step=len(login_success)
            )
            is_webftp = self.is_webftp(um[0])
            self.add_event(
                person=person.person,
                name='first_login',
                obj_type="user",
                obj_value=user,
                event_type='once_person_object',
                value=0,
                metadata={'type': 'ftp', 'webftp': is_webftp},
            )

    def ssh_shell_counter(self, matches):

        for user, um in self.matches_by_field('user', matches).items():
            person = self.customer_info.person_by_user(user)
            self.metrics_state.counter(
                'user_login',
                tags=['person', 'user', 'type', 'result'],
                tag_values=[person.person, user, 'ssh', 'success'],
                step=len(um)
            )
            self.add_event(
                person=person.person,
                name='first_login',
                obj_type="user",
                obj_value=user,
                event_type='once_person_object',
                value=1,
                metadata={'type': 'ssh'},
            )

    def sftp_counter(self, matches):
        for user, um in self.matches_by_field('user', matches).items():
            person = self.customer_info.person_by_user(user)
            self.metrics_state.counter(
                'user_login',
                tags=['person', 'user', 'type', 'result'],
                tag_values=[person.person, user, 'sftp', 'success'],
                step=len(um)
            )

            is_webftp = self.is_webftp(um[0])
            self.add_event(
                person=person.person,
                name='first_login',
                obj_type="user",
                obj_value=user,
                event_type='once_person_object',
                value=0,
                metadata={'type': 'sftp', 'webftp': is_webftp},
            )
