from datetime import datetime
import arrow

# all functions in here returning datetime objects should be
# returning 'aware' UTC


def unixtime_to_dt(time):
    if isinstance(time, str):
        time = int(time)
    return arrow.get(time)


def utc_to_dt(time):
    try:
        local = datetime.strptime(
            time, "%Y-%m-%d %H:%M:%S")
        return arrow.get(local, 'UTC')
    except Exception as err:
        pass


def pst_to_dt(time):
    try:
        local = datetime.strptime(
            time, "%Y-%m-%d %H:%M:%S")
        return arrow.get(local, 'US/Pacific').to('utc')
    except Exception as err:
        pass


def utcnow():
    return arrow.utcnow()


def utc_yesterday():
    return utcnow().shift(days=-1)


def utc_tomorrow():
    return utcnow().shift(days=1)


def pacific_now():
    return arrow.utcnow().to('US/Pacific')


def day_range(start, end):
    return [r for r in arrow.Arrow.range('day', start, end)]


def seconds_since(start, end):
    return (start - end).total_seconds()


def apache_stamp_to_iso(stamp):
    # 27/Jan/2020:10:57:28 -0800
    fmt = '%d/%b/%Y:%H:%M:%S %z'
    return arrow.Arrow.strptime(
        stamp, fmt).isoformat()


def apache_stamp_to_arrow(stamp):
    # 27/Jan/2020:10:57:28 -0800
    fmt = '%d/%b/%Y:%H:%M:%S %z'
    return arrow.Arrow.strptime(
        stamp, fmt)
