
import signal
from cement import App, TestApp, init_defaults
from cement.core.exc import CaughtSignal
from .core.exc import PlowError
from .controllers.base import Base

# configuration defaults
CONFIG = init_defaults('plow')

CONFIG['plow']['exporter_auth_token'] = False
CONFIG['plow']['exporter_host'] = "127.0.0.1"
CONFIG['plow']['exporter_port'] = 8902
CONFIG['plow']['metrics_path'] = '/var/lib/metrics/plow'
CONFIG['plow']['state_path'] = '/var/lib/plow/state'
CONFIG['plow']['event_api_token'] = '1234'
CONFIG['plow']['event_api_url'] = "https://local.dhops.com:YOURPORT"
CONFIG['plow']['state_api_token'] = '1234'
CONFIG['plow']['state_api_url'] = "https://local.dhops.com:YOURPORT"
CONFIG['plow']['fast_scrape'] = False
CONFIG['plow']['disable_metrics'] = ""


class Plow(App):
    """Plow primary application."""

    class Meta:
        label = 'plow'

        # configuration defaults
        config_defaults = CONFIG

        # call sys.exit() on close
        exit_on_close = True

        # load additional framework extensions
        extensions = [
            'yaml',
            'colorlog',
            'jinja2',
        ]

        # configuration handler
        config_handler = 'yaml'

        # configuration file suffix
        config_file_suffix = '.yml'

        # set the log handler
        log_handler = 'colorlog'

        # set the output handler
        output_handler = 'jinja2'

        # register handlers
        handlers = [
            Base
        ]


class PlowTest(TestApp, Plow):
    """A sub-class of Plow that is better suited for testing."""

    class Meta:
        label = 'plow'


def main():
    with Plow() as app:
        try:
            app.run()

        except AssertionError as e:
            print('AssertionError > %s' % e.args[0])
            app.exit_code = 1

            if app.debug is True:
                import traceback
                traceback.print_exc()

        except PlowError as e:
            print('PlowError > %s' % e.args[0])
            app.exit_code = 1

            if app.debug is True:
                import traceback
                traceback.print_exc()

        except CaughtSignal as e:
            # Default Cement signals are SIGINT and SIGTERM, exit 0 (non-error)
            print('\n%s' % e)
            if e.signum == signal.SIGTERM:
                app.exit_code = 0
                app.close()


if __name__ == '__main__':
    main()
