import os
import sys
import arrow
import jsonpickle
from abc import ABC


def pacific_now():
    return arrow.utcnow().to('US/Pacific')


def create_state(filename):
    if not os.path.exists(filename):
        with open(filename, 'w') as f:
            data = dict()
            data['state_created_stamp'] = pacific_now().replace(
                hour=00, minute=00, second=00, microsecond=00)
            encoded = jsonpickle.encode(data, keys=True)
            f.write(encoded)
            f.close()


class JSONState(ABC):
    def __init__(self, name, path=None, rotate=1, flag='r'):
        self.name = name
        self.rotate_days = rotate
        self.state_path = path or os.getenv(
            'STATE_PATH', '/tmp/state')
        self.state_filename = os.path.join(self.state_path, name + '.json')
        create_state(self.state_filename)

    def read(self):
        state_date = dict()
        try:
            with open(self.state_filename, 'r') as f:
                state_data = jsonpickle.decode(f.read(), keys=True)
                f.close()
        except Exception as e:
            return dict()

        return state_data

    def write(self, data):
        with open(self.state_filename, 'w') as f:
            f.write(jsonpickle.encode(data, keys=True))
            f.close()
        return data

    def rotate(self):
        if not self.rotate_days:
            return True

        should_rotate = 0
        try:
            state_db = self.read()
            if self.get('state_created_stamp'):
                created_time = self.get('state_created_stamp')
                now_time = pacific_now()
                if (now_time - created_time).days >= self.rotate_days:
                    should_rotate = 1
            else:
                should_rotate = 1
        except Exception as e:
            pass

        if should_rotate:
            os.remove(self.state_filename)
            # write blank dict
            create_state(self.state_filename)

    def get(self, key):
        state = self.read()
        if key in state:
            return state[key]

    def set(self, key, value):
        state = self.read()
        state[key] = value
        self.write(state)

    def delete(self, key):
        state = self.read()
        del state[key]
        self.write(state)
